function [ResEDF, ResUserData, Replace] = am2(EDF, UserData, Opt)
% template for a plugin function
% a plugin *must* set the ResEDF values for:
%   Head.NS, Head.NRec, Head.PhysMin, Head.PhysMax, Labels

% Changes:
% 09/18/98 : data format changed

if nargin < 3
  Opt = '';
end;

Replace=0;         % a tribute to viewedf2

switch Opt
  case 'Reset'
    % Check whether EDF File has changed, update UserData, ...
    [ResEDF, ResUserData] = LocalReset(EDF, UserData);
  case 'Menu'
    % set UserData if necessary
    [ResEDF, ResUserData] = LocalConfigure(EDF, UserData);
  otherwise
    % Update ResEDF
    [ResEDF, ResUserData] = LocalUpdate(EDF, UserData);
end;
return;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ResEDF, ResUserData] = LocalConfigure(EDF, UserData)    % channel selection only
[UserDat]=channels(UserData,2);
if ~UserDat.keepold
   UserData=UserDat;
end;
Fs=EDF.Head.SampleRate(1);
[N,Wn]=buttord([UserData.sp.lowerco,UserData.sp.upperco]*2/Fs,...
   [UserData.sp.lowerco-3,UserData.sp.upperco+3]*2/Fs,3,20);    %determine the appropriate filter order
[UserData.Spindle.B UserData.Spindle.A]=butter(N,[UserData.sp.lowerco,UserData.sp.upperco]*2/Fs);   % Spindel-BandPass
[UserData.Slow.B UserData.Slow.A]=butter(3,[1]*2/Fs);             % Slow-Wave-LowPass
[UserData.Delta.B UserData.Delta.A]=butter(3,[1,4]*2/Fs);         % Delta-Wave-BandPass
[UserData.Glaett.B UserData.Glaett.A]=cheby1(3,0.1,[1/(UserData.sp.smoowinlen*0.5)]*2/Fs);     % smoothing-filter, not recommended by Amzica & Steriade
[UserData.Kcomp.B]=template(EDF.Head.SampleRate(1),UserData.kc.template);
[H,W]=freqz(UserData.Kcomp.B,1);
UserData.Kcomp.A=abs(max(H));
[ResEDF, ResUserData] = LocalUpdate(EDF, UserData);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ResEDF, ResUserData] = LocalReset(EDF, UserData)
UserData.select=zeros(4,1);
UserData.kc.template=1;
UserData.kc.output=3;
UserData.sp.smoowinlen=2;
UserData.sp.upperco=15;
UserData.sp.lowerco=12;
UserData.sp.logarithmic=0;
[UserDat]=channels(UserData,2);    % channel selection
if ~UserDat.keepold
   UserData=UserDat;
end;
Fs=EDF.Head.SampleRate(1);
[N,Wn]=buttord([UserData.sp.lowerco,UserData.sp.upperco]*2/Fs,...
   [UserData.sp.lowerco-3,UserData.sp.upperco+3]*2/Fs,3,20);    %determine the appropriate filter order
[UserData.Spindle.B UserData.Spindle.A]=butter(N,[UserData.sp.lowerco,UserData.sp.upperco]*2/Fs);   % Spindel-BandPass
[UserData.Slow.B UserData.Slow.A]=butter(3,[1]*2/Fs); % Slow-Wave-LowPass
[UserData.Delta.B UserData.Delta.A]=butter(3,[1,4]*2/Fs);         % Slow-Wave-LowPass
[UserData.Kcomp.B]=template(EDF.Head.SampleRate(1),UserData.kc.template);
[H,W]=freqz(UserData.Kcomp.B,1);
UserData.Kcomp.A=abs(max(H));
[UserData.Glaett.B UserData.Glaett.A]=cheby1(3,0.1,[1/(UserData.sp.smoowinlen*0.5)]*2/Fs); % smoothing-filter, not recommended by Amzica & Steriade
[ResEDF, ResUserData] = LocalUpdate(EDF, UserData);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ResEDF, ResUserData] = LocalUpdate(EDF, UserData)
% This function returns the first channel of the first EDF file
[ResEDF,ResUserData]=berechnung(EDF, UserData);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Version 0.70
%	20.08.1998
%	Copyright (c) by  Alois Schloegl
%	a.schloegl@ieee.org	
% Init
function [ResEDF,ResUserData]=berechnung(EDF, UserData);

% Berechnung von K-Komplexen, Spindeln, Slow-Waves und Delta nach Amzica, Steriade 1997

chansel=[1:7];    % hopefully, Siesta-Data is given...
k=cell2struct(EDF.Record(chansel),'a',length(chansel));  % convert for calculation
for t=1:length(chansel)
   S(t,:)=k(t).a';                      % build matrix (found no command)
end;                                   % UserData required for the Display-Option

ResEDF=EDF;                            % if nothing is selected, nothing should be done
ResUserData=UserData;
ResEDF.Head.NS=0;                      % up to now
count=0;                               % counter for calculated channels
laenge=size(S);                   % cell-dimensions
r=S;

if UserData.select(1)                   % K-Komplex
   if laenge(2)<3*length(UserData.Kcomp.B) % segment is to short to use the filtfilt operation
      h=errordlg('The displayed segment is to short!','K-complex error...');
      waitfor(h);
   else      
   for i=chansel   
      count=count+1;                   % next channel
      v=filtfilt(UserData.Kcomp.B,UserData.Kcomp.A,S(i,:));      % filtering
      ResEDF.Record(count)=num2cell(v.^UserData.kc.output,2);    % put into output
      ResEDF.Head.Label(count,:)=['K-comp.: ',EDF.Head.Label(i,1:7)];
      factor=(UserData.kc.output);
      if UserData.kc.output~=2
         ResEDF.Head.PhysMin(1:count)=(EDF.Head.PhysMin(i)^factor)/UserData.kc.template;
      else
         ResEDF.Head.PhysMin(1:count)=0;
      end;
      ResEDF.Head.PhysMax(1:count)=(EDF.Head.PhysMax(i)^factor)/UserData.kc.template;
      ResEDF.Head.PhysDim(count,:)=double(['V^',num2str(UserData.kc.output),'    ']);
   end;
   S=r;                                % restore original signal
   end;
end;
if UserData.select(2)                  % Spindles
   for i=chansel   
      count=count+1;                   % next channel
      S(i,:)=filtfilt(UserData.Spindle.B,UserData.Spindle.A,S(i,:));     % filtering
      S(i,:)=filtfilt(UserData.Glaett.B,UserData.Glaett.A,S(i,:).^2);  % smoothing
      if UserData.sp.logarithmic
         S(i,:)=log10(1+S(i,:));
         ResEDF.Head.PhysMax(count)=log10(1+EDF.Head.PhysMax(i)*2);
      else
         ResEDF.Head.PhysMax(count)=EDF.Head.PhysMax(i)*2;
      end;
      ResEDF.Record(count)=num2cell(S(i,:),2);    % put into output
      ResEDF.Head.Label(count,:)=['Spindle: ',EDF.Head.Label(i,1:7)];
      ResEDF.Head.PhysMin(count)=0;
      ResEDF.Head.PhysDim(count,:)=double('V (log)');
   end;
   S=r;
end;
if UserData.select(3)                  % Slow-Waves
   for i=chansel
      count=count+1;                   % next channel
      S(i,:)=filtfilt(UserData.Slow.B,UserData.Slow.A,S(i,:));     % filtering
      ResEDF.Record(count)=num2cell(S(i,:),2);    % put into output
      ResEDF.Head.Label(count,:)=['SlowWave: ',EDF.Head.Label(i,1:6)];
      ResEDF.Head.PhysMin(count)=EDF.Head.PhysMin(i);
      ResEDF.Head.PhysMax(count)=EDF.Head.PhysMax(i);
      ResEDF.Head.PhysDim(count,:)=double('V      ');
   end;
   S=r;
end;
if UserData.select(4)                  % Delta
   for i=chansel
      count=count+1;                   % next channel
      S(i,:)=filtfilt(UserData.Delta.B,UserData.Delta.A,S(i,:));     % filtering
      ResEDF.Record(count)=num2cell(S(i,:),2);    % put into output
      ResEDF.Head.Label(count,:)=['Delta: ',EDF.Head.Label(i,1:9)];
      ResEDF.Head.PhysMin(count)=EDF.Head.PhysMin(i);
      ResEDF.Head.PhysMax(count)=EDF.Head.PhysMax(i);
      ResEDF.Head.PhysDim(count,:)=double('V      ');
   end;
end;
ResEDF.Head.NS=count;                % total number of calculated signals...
ResEDF.Head.NRec=EDF.Head.NRec;
ResUserData=UserData;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [y]=template(Fs,mode);

% [y]=lengtemp(Fs,mode);
% Fs is the SampleRate, mode=2: Richards's template, otherwise Jobert's template
% y the calculated template

j=Fs;
if mode==2   % template of Richard/Lengelle 1998
   k=0:14;      % defined duration: 1.75 sec in Richard/Lengelle dt=0.125sec.
   Fs=round(Fs*1.75);
   s(k+1)=exp((-(k-7).^2)/3.2);
   s(k+1)=s(k+1).*cos(0.5*pi*k);
   y=interp1(k,s(k+1),1/Fs:15/Fs:15);
   r=max(y);
   y=-y/r;
   y(isnan(y))=0;
else         % template of Jobert 1991
   load kcomplex;
   y=te;
   Fsbase=100;
   y=resample(y,round(Fs),Fsbase);
end;
%h=figure;         % Template
%plot(y);
%text=['K-complex template of '];
%if mode~=2
%   text=[text,'Jobert, Fs=',num2str(j)];
%else
%   text=[text,'Richard, Fs=',num2str(j)];
%end;
%title(text);
%xlabel('Samples [time]');
%ylabel('Amplitude [arbitrary units]');
%zoom;
%pause;
%close(h);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LocalSelectChannels
% select channels to be displayed on the screen
function [UserDaten]=channels(UserDaten,what)

% what=1: select all
% what=0: unselect all
% what=2: no change

UserDaten.keepold=0;
chans=1:6;
if what~=2
   UserDaten.select(chans)=what;     % pre-set
end;
% find selected channels
text=['Select the various calculations performed...'];
dlgh = dialog(...
    'Name', text, ...
    'CloseRequestFcn', 'set(gcf,''UserData'',''Cancel'');uiresume;');
cbx = 0.1;
cby = 0.95;
cbxspace = 0.4;
cbyspace = 0.8 / 2 + 0.07; 
cbwidth = 0.35;
cbheight = LocalGetFontHeight;

% display EDF channels
text=['K-complex';'Spindles ';'Slow-Wave';'Delta    '];
for i = 1:4
  uih(i) = uicontrol(dlgh, ...
    'Style', 'CheckBox', ...
    'Units', 'Normalized', ...
    'String', sprintf('Mode : %s', text(i, :)), ...
    'Value', UserDaten.select(i), ...
    'Position', [cbx, cby - ((i-1)/2)*cbyspace, ...
       cbwidth, cbheight]);
end;
uicontrol(dlgh, ...
    'Style', 'Text', ...
    'Units', 'Normalized', ...
    'String', {'KC-Template:'}, ...
    'HorizontalAlignment','left',...
    'Position', [cbx+cbxspace, cby, cbwidth/2, cbheight]);
template=uicontrol(dlgh, ...
    'Style', 'popupmenu', ...
    'Units', 'Normalized', ...
    'String', {'Jobert','Richard'}, ...
    'Value', UserDaten.kc.template,...
    'Position', [cbx+cbxspace+cbwidth/2, cby, cbwidth/2, cbheight]);
output=uicontrol(dlgh, ...
    'Style', 'popupmenu', ...
    'Units', 'Normalized', ...
    'String', {'linear','squared (^2)','cubic (^3)'}, ...
    'Value', UserDaten.kc.output,...
    'Position', [cbx+cbxspace+cbwidth/2, cby-1/4*cbyspace, cbwidth/2, cbheight]);
uicontrol(dlgh, ...
    'Style', 'Text', ...
    'Units', 'Normalized', ...
    'HorizontalAlignment','left',...
    'String', {'Display Output:'}, ...
    'Position', [cbx+cbxspace, cby-1/4*cbyspace, cbwidth/2, cbheight]);
 
uicontrol(dlgh, ...
    'Style', 'Text', ...
    'Units', 'Normalized', ...
    'String', {'Lower Cut-off: (Hz)'}, ...
    'HorizontalAlignment','left',...
    'Position', [cbx+cbxspace, cby-1/2*cbyspace, cbwidth/2, cbheight]);
lowerco=uicontrol(dlgh, ...
    'Style', 'edit', ...
    'Units', 'Normalized', ...
    'Horizontalalignment','left',...
    'String', num2str(UserDaten.sp.lowerco),...
    'Position', [cbx+cbxspace+cbwidth/2, cby-1/2*cbyspace, cbwidth/2, cbheight*1.2]);
upperco=uicontrol(dlgh, ...
    'Style', 'edit', ...
    'Units', 'Normalized', ...
    'Horizontalalignment','left',...
    'String', num2str(UserDaten.sp.upperco),...
    'Position', [cbx+cbxspace+cbwidth/2, cby-1/2*cbyspace-2*cbheight, cbwidth/2, cbheight*1.2]);
uicontrol(dlgh, ...
    'Style', 'Text', ...
    'Units', 'Normalized', ...
    'HorizontalAlignment','left',...
    'String', {'Upper Cut-off: (Hz)'}, ...
    'Position', [cbx+cbxspace, cby-1/2*cbyspace-2*cbheight, cbwidth/2, cbheight]);
smoowinlen=uicontrol(dlgh, ...
    'Style', 'popupmenu', ...
    'Units', 'Normalized', ...
    'String', {'0.5 sec.','1 sec.','1.5 sec.','2 sec.','2.5 sec.','3 sec.'}, ...
    'Value', UserDaten.sp.smoowinlen,...
    'Position', [cbx+cbxspace+cbwidth/2, cby-1/2*cbyspace-4*cbheight, cbwidth/2, cbheight]);
uicontrol(dlgh, ...
    'Style', 'Text', ...
    'Units', 'Normalized', ...
    'HorizontalAlignment','left',...
    'String', {'Smoothing:'}, ...
    'Position', [cbx+cbxspace, cby-1/2*cbyspace-4*cbheight, cbwidth/2, cbheight]);
 
 logarithmic=uicontrol(dlgh, ...
    'Style', 'RadioButton', ...
    'Units', 'Normalized', ...
    'HorizontalAlignment','left',...
    'String', {'Logarithmic output'}, ...
    'Value',UserDaten.sp.logarithmic,...
    'Position', [cbx+cbxspace, cby-1/2*cbyspace-6*cbheight, cbwidth, cbheight]);
uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'Cancel', ...
    'Position', [0.1, 0.02, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''Cancel'');uiresume;');
uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'OK', ...
    'Position', [0.7, 0.02, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''OK'');uiresume;');
 uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'Unselect', ...
    'Position', [0.1, 0.08, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''Unselect'');uiresume;');
 uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'Select', ...
    'Position', [0.7, 0.08, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''Select'');uiresume;');
uiwait(dlgh);
changed = 0;
hilfe=get(dlgh,'UserData');
UserDaten.kc.template=get(template,'Value');
UserDaten.kc.output=get(output,'Value');
UserDaten.sp.smoowinlen=get(smoowinlen,'Value');
UserDaten.sp.logarithmic=get(logarithmic,'Value');
lowerco=str2num(get(lowerco,'String'));
if ~isempty(lowerco)                      % check for input errors
   if lowerco(1)<=4
      lowerco(1)=4;                       % Minimum selectable frequency
   end;
   if lowerco(1)>=30
      lowerco(1)=30;                      % Maximal lower bound of filter selectable frequency
   end;
   UserDaten.sp.lowerco=lowerco(1);
end;
upperco=str2num(get(upperco,'String'));
if ~isempty(upperco)                      % check for input errors
   if upperco(1)<=5
      upperco(1)=5;                       % Minimum selectable frequency
   end;
   if upperco(1)<=lowerco(1)              % upper cut-off lower than lower cut-off
      upperco(1)=lowerco(1)+1;
   end;
   if upperco(1)>=32
      upperco(1)=32;                      % Maximum upper bound for selectable frequency
   end;
   UserDaten.sp.upperco=upperco(1);
end;
UserDaten.sp.smoowinlen=get(smoowinlen,'Value');
if strcmp(hilfe,'OK')
   changed = 1;
   for i=1:4
        UserDaten.select(i)=get(uih(i),'Value'); 
  end;
end; 
if strcmp(hilfe,'Cancel')
   UserDaten.keepold=1; 
end; 
if strcmp(hilfe,'Unselect')
   delete(dlgh);
   [UserDaten]=channels(UserDaten,0);   %unselect all
   return;
elseif strcmp(hilfe,'Select')
   delete(dlgh);
   [UserDaten]=channels(UserDaten,1);   %select all
   return;
end;
delete(dlgh);
if ~(sum(UserDaten.select)) & (~UserDaten.keepold)               
   text=['No calculation selected!'];
   h=errordlg(text, 'Warning');
   waitfor(h);
end;
return;

% LocalGetFontHeight
% get Fontheight
function hght=LocalGetFontHeight()
tempH = uicontrol(...
    'Style', 'Text', ...
    'String', 'Gg', ...
    'Units', 'Normalized', ...
    'FontUnits', 'Normalized', ...
    'Position', [0, 0, 1, 1], ...
    'Visible', 'off');
hght = get(tempH, 'FontSize') * 1.25; % 1.25 makes things look better
delete(tempH);
return;

